function [ labels ] = authorityShift_point( points, iparam)
% authority-shift algorithm
% Jan 2009 by Minsu Cho
% revised May 2010 by Minsu Cho

iparam.authorityshift_itermax = 10000;
iparam.authorityshift_nseed = 0; % not used in current version
iparam.authorityshift_thresholdKL = 0.05; % not used in current version

Brw = iparam.Prw;
c = iparam.authorityshift_c;
iter_max = iparam.authorityshift_itermax;

bShow = 0;  % show each layer one by one
bShowDetail = 0;
bUnbiased = 1; % ignore the final jump at PPR score
dirShift = 1; % 1: forward  2: backward
bReduce = 1;  % supernode aggregation


[V_PR, seedList_initial, szOfSeedList_initial, authIdx_all_initial, W_PPR_initial] = ...
    extractPeaks( Brw, c, bUnbiased, iparam.authorityshift_nseed, iparam.authorityshift_thresholdKL);
%layerStability = [];    layerNC = [];
%hierCluster{1,1} = [ 1:nSize; authIdx_all_initial ];
authIdx_initial = seedList_initial;% enforce that seeds stick to themselves

% V_PR : nNode by 1 pageRank vector
% seedList_initial : indexes of seed nodes
% W_PPR : nSeed_initial by nNode

if bShow % for visualization
    spRow = 3; spCol = 5;
    margin_ratio=0.07;
    minX = min(points(1,:));    maxX = max(points(1,:));
    minY = min(points(2,:));    maxY = max(points(2,:));
    marginX = (maxX-minX)*margin_ratio; marginY = (maxY-minY)*margin_ratio;
    % visualize the initial data
    figure; clf; spIdx = 1;   
    subplot(spRow,spCol,spIdx,'replace');
    plot2Dpoints_with_clusters_AP(points,authIdx_all_initial);
    axis equal
    xlim([minX-marginX maxX+marginX]);
    ylim([minY-marginY maxY+marginY]);
    title(sprintf('%d clsts (data:%d)',length(unique(authIdx_all_initial)),length(V_PR)));  
end

% seedList_initial    : 1 by nSeed - initial seed list
% szOfSeedList_initial: 1 by nSeed - num of elements sharing the initial supernodes(seeds)
% authIdx_all_initial : 1 by nSize - authority nodes of each elements 

% make column-stochastic, but already so
%W_PPR = W_PPR*diag(1./sum(W_PPR,1)); 

% initiaization for authority-shift
W_PPR_reduced = W_PPR_initial; % initial PPRs for the current layer
W_PPR_nOrder = W_PPR_initial;  % nth order PPRs in the current layer
%seedList = seedList_initial;
%layerNC = [ length(seedList_initial) ];
authIdx = seedList_initial; % current index set of authority nodes (super nodes)
szOfAuthIdx = szOfSeedList_initial;% member size for each super node
authIdx_all = authIdx_all_initial; % current authority index for each initial node
nPPROrder = 1; % PPR order for the current layer

for kIter = 1:iter_max
    % authority-shifting: where is my authority node?
    [ shiftVal, elementIdx ] = max(W_PPR_nOrder,[],dirShift);
    
    % find authority-sink for each node
    sink_elementIdx = find_sink( elementIdx );
    final_authIdx = authIdx(sink_elementIdx);
    
    shifted_idx = find(final_authIdx ~= authIdx);
    new_seedList = sort(unique(final_authIdx));

    % if the num of cluster is less then k 
%     if length(new_seedList) < iparam.k %|| nStable > pprThresold
%         kDelShift = iparam.k - length(new_seedList); % eliminate the weakest k shifts from the layer
%         fprintf('* Total aggregation %d :prevent the weakest %d shifts from the layer\n',kDelShift);
%         
%         [ tmp sh_idx ] = sort(shiftVal(shifted_idx),'ascend');
%         shifted_idx = setdiff(shifted_idx,shifted_idx(sh_idx(1:kDelShift)));
%     end
    
    if bReduce
       %% hierarchical approach 
        if isempty(shifted_idx)
            % PPR propagation for n-th order PPR
            W_PPR_nOrder = W_PPR_reduced*W_PPR_nOrder; 
            nPPROrder = nPPROrder + 1;
        else % supernode aggregation and matrix reduction
            % edit authIdx_all
            for cIdx = shifted_idx
                authIdx_all( authIdx_all == authIdx(cIdx) ) = final_authIdx(cIdx);
            end
            % start updating W_PPR_reduced
            tmp_W_PPR_reduced = zeros(length(authIdx),length(new_seedList));
            new_seedList = sort(unique(final_authIdx));
            new_szOfAuthIdx = zeros(1,length(new_seedList));
            %new_szSeedList = zeros(length(new_seedList),1);
            
            % First, row-wise summation considering sizes of supernodes
            %tmp_c = 0;
            for jIter = 1:length(new_seedList)
                tmp_seedListIdx = find(final_authIdx == new_seedList(jIter));
                new_szOfAuthIdx(jIter) = sum(szOfAuthIdx(tmp_seedListIdx));
                %tmp_c = tmp_c + new_szOfAuthIdx(jIter);
                if length(tmp_seedListIdx)>1
                    tmp_Norm = diag(szOfAuthIdx(tmp_seedListIdx))./new_szOfAuthIdx(jIter);
                    tmp_W_PPR_reduced(:,jIter) = sum( W_PPR_reduced(:,tmp_seedListIdx) * tmp_Norm , 2);
                    %tmp_W_PPR_reduced(:,jIter) = sum( W_PPR_nOrder(:,tmp_seedListIdx) * tmp_Norm , 2);
                else
                    tmp_W_PPR_reduced(:,jIter) = W_PPR_reduced(:,tmp_seedListIdx);
                    %tmp_W_PPR_reduced(:,jIter) = W_PPR_nOrder(:,tmp_seedListIdx);
                end
            end
            %if tmp_c ~= nSeed_initial,  disp('sum err1!'); pause; end
            
            % Second, column-wise summation, just sum!
            new_W_PPR_reduced = zeros(length(new_seedList),length(new_seedList));
            for jIter = 1:length(new_seedList)
                tmp_seedListIdx = find(final_authIdx == new_seedList(jIter));
                new_W_PPR_reduced(jIter,:) = sum( tmp_W_PPR_reduced(tmp_seedListIdx,:) , 1);
            end
%             for jIter = 1:length(new_seedList)
%                 new_W_PPR_reduced(jIter,jIter) = 0;
%             end
            % make column-stochastic, but already so
            %new_W_PPR_reduced = new_W_PPR_reduced * diag( 1./sum(new_W_PPR_reduced,1) );            
            W_PPR_reduced = new_W_PPR_reduced;  W_PPR_nOrder = new_W_PPR_reduced;
            seedList = new_seedList;    authIdx = new_seedList; szOfAuthIdx = new_szOfAuthIdx;
            
            if bShow % for visualization
                spIdx = spIdx + 1;  
                subplot(spRow,spCol,spIdx,'replace');
                plot2Dpoints_with_clusters_AP(points,authIdx_all,0);%,[initial_authIdx_all_seed; authIdx_initial]);
                axis equal
                xlim([minX-marginX maxX+marginX]);
                ylim([minY-marginY maxY+marginY]);
                nPrevAuth = length(authIdx)+1;            
                title(sprintf('#%d: %d cls aft %d (%.1f)',kIter,length(unique(seedList)),nPPROrder,...
                    (nPPROrder-1)*nPrevAuth*log(nPrevAuth)));
                if spIdx >= spRow*spCol
                    break;
                end
            end
            %layerStability = [ layerStability nPPROrder ];
            %layerNC = [ layerNC length(unique(seedList)) ];
            nPPROrder = 1; % reset the PPR order
        end
        
    else
        %% basic authority-shift approach
        if isempty(shifted_idx)
            % PPR propagation for n-th order PPR
            W_PPR_nOrder = W_PPR_reduced*W_PPR_nOrder; 
            nPPROrder = nPPROrder + 1;
        else
            %edit authIdx_all
            for cIdx = shifted_idx
                authIdx_all( authIdx_all == authIdx(cIdx) ) = final_authIdx(cIdx);
            end
            if bShow %&& kIter > 30
                spIdx = spIdx + 1;  
                subplot(spRow,spCol,spIdx,'replace');
                plot2Dpoints_with_clusters_AP(points,authIdx_all,[initial_authIdx_all_seed; authIdx_initial]);
                axis equal
                xlim([minX-marginX maxX+marginX]);
                ylim([minY-marginY maxY+marginY]);
                title(sprintf('#%d: %d clsts(%d) after %d',kIter,length(unique(final_authIdx)),length(unique(authIdx_all)),nStable));
                if spIdx >= spRow*spCol
                    break;
                end
            end
            authIdx = final_authIdx;
            nPPROrder = 1; % reset the PPR order
        end
    end
    
    % stop if the numb of cluster is equal to or less then k 
    if length(authIdx) <= iparam.k %|| nStable > pprThresold
        break;
    end
       
end

labels = authIdx_all; % return the final result

fprintf('authority-shift: %d order done - the number of clusters %d\n',kIter, length(authIdx));
if bShow && bShowDetail
    draw_plot_2( [ layerStability NaN], [ 1:length(layerStability)+1 ], {'Stability'})
    draw_plot_3( layerNC, [1:length(layerNC)], {'Number of clusters'})
end

