function [ pageRank, seedList_initial, szOfSeedList_initial, authIdx_all_initial, W_PPR_reduced_initial ] = extractPeaks( Brw, c, bUnbiased, nSeeding, thresholdKL )
% extract local peaks from the matrix
%
% method 'radius' : local radius suppression (param: radius)
% method 'gradient': local radius suppression

if nargin < 2
    method = 'PPV';
end
bReInitialization = 1;
bUnbiased = 1;
nSize = size(Brw,1);
seedList_initial = [];
nSeed_initial = 0;
% PageRank vector
%tic;
pageRank = ( (speye(nSize) - c*Brw) \ ones(nSize,1) ) .* ((1-c)/nSize);
%fprintf('PageRank calculation done ... %f seconds\n',toc);
% showLandscape(points,V_PR);
% title('Overall PageRank');

if nSeeding == 0
    seedList_initial = 1:nSize;
    nSeed_initial = nSize;
    % make personalization matrix
    M_pers = speye(nSize);
    %T_seedIdxPR = sparse(1,nSize);
    %tic;
    W_PPR = ( (speye(nSize) - c*Brw) \ M_pers ).* (1-c);% nSize by nSeed
    if bUnbiased
        W_PPR = W_PPR - M_pers .* (1-c); % bias subtraction
        normM = diag( 1./sum(W_PPR,1) );
        W_PPR = W_PPR * normM;
        pageRank = ( pageRank - ones(nSize,1) .* ((1-c)/nSize) );
        pageRank = pageRank./sum(pageRank);
    end
    szOfSeedList_initial = ones(length(seedList_initial),1); 
    authIdx_all_initial = 1:nSize;
    W_PPR_reduced_initial = W_PPR;
    %fprintf('%d Partial PPR caculation done ... %f seconds\n',nSeed_initial, toc); 
    return
end

%tic;
W_PPR = sparse(nSize, nSeeding);
acPPR = zeros(nSize, 1);

redisualPageRank  = pageRank ;
dKL = Inf;
iIter = 0;

historyKLD = [];%[ KLDiv( pageRank', ones(1,nSize)./nSize ) ]; 
%suppress non- local maxima
while( dKL > thresholdKL && iIter < nSeeding )
    iIter = iIter + 1;
    [ tmp idxPeak ] = max(redisualPageRank);
    seedList_initial = [ seedList_initial; idxPeak ];
    
    M_pers = sparse(nSize,1); M_pers(idxPeak) = 1;
    PPR = ( (speye(nSize) - c*Brw) \ M_pers ).* (1-c);% nSize by nSeed
    if bUnbiased
        PPR = PPR - M_pers .* (1-c); % bias subtraction
        PPR = PPR ./ sum(PPR);
    end
    W_PPR(:,iIter) = PPR;
    acPPR = acPPR + PPR;
    dKL = KLDiv(pageRank',acPPR');
    historyKLD = [ historyKLD dKL ];
    %fprintf('iter%d: KLD %f\n',iIter,dKL ); 
    redisualPageRank = pageRank - ( acPPR./ iIter); 
%     showLandscape(points,redisualPageRank);
%     title([ 'PageRank Residual #' num2str(iIter) ]);
    redisualPageRank( seedList_initial ) = -Inf;
end  
seedList_initial = seedList_initial';
nSeed_initial = length(seedList_initial);
%draw_plot_1( historyKLD, [1:nSeed_initial], {'Seed extraction by PageRank Residual'})

%fprintf('%d Partial PPR seeding done ... %f seconds\n',iIter, toc); 


% seed propagation to unlabel elements
if nSize > nSeed_initial
    if bShow
        figure;
        showLandscape_vis1(points,V_PR,seedList_initial);
        axis equal
        xlim([minX-marginX maxX+marginX]);
        ylim([minY-marginY maxY+marginY]);
    end
    % seeded allocation for unlabeled
    %[ tmp, elementIdx ] = max(repmat(V_PR,1,size(W_PPR,2)).*W_PPR,[],2);% nSize by 1
    [ tmp, elementIdx ] = max(W_PPR,[],2);% nSize by 1 
    authIdx_all_initial =  seedList_initial(elementIdx);% initial label for each element
    szOfSeedList_initial = zeros(length(seedList_initial),1);
    for kIter = 1:nSeed_initial
        szOfSeedList_initial(kIter) = sum( authIdx_all_initial == seedList_initial(kIter) );
    end
else
    szOfSeedList_initial = ones(length(seedList_initial),1); 
    authIdx_all_initial = 1:nSize;
end

% compute real PPR matrix for each supernode considering all its elements 
if nSize > nSeed_initial && bReInitialization
    %make personalization matrix, again!
    M_pers = sparse(nSize,nSeed_initial);
    %tic;
    for kIter = 1:nSeed_initial
        idxForSeed = find( seedList_initial(kIter) == authIdx_all_initial);
        M_pers(idxForSeed,kIter) = 1/length(idxForSeed);
        %M_pers_size(kIter) = length(idxForSeed);
        %T_authIdx2kPR(seedList(kIter)) = kIter;
    end
    W_PPR = ( (speye(nSize) - c*Brw) \ M_pers ).* (1-c); % nSize by nSeed
    if bUnbiased
        W_PPR = ( W_PPR - M_pers.* (1-c) )./c; % bias subtraction
        normM = diag( 1./sum(W_PPR,1) );
        W_PPR = W_PPR * normM;
    end
    %fprintf('%d Partial PPR caculation done, again ... %f seconds\n',nSeed_initial, toc);
    %tic
    % initial aggregation by PPR summation!   
    W_PPR_reduced_initial = zeros(nSeed_initial,nSeed_initial);
    for kIter = 1:nSeed_initial
        idxForSeed = find( seedList_initial(kIter) == authIdx_all_initial);
        W_PPR_reduced_initial(kIter,:) = sum(W_PPR(idxForSeed,:),1);
    end
    %fprintf('initial aggregation ... %f seconds\n',toc);
else
    W_PPR_reduced_initial = W_PPR;
end