function [data,truelabels] = build_scene(caseid)
% [data,size_cluster] = build_scene(case)
%
% Minsu Cho 2009, extended from codes of Jianbo Shi, 1997

if caseid == 1,
    npts1 = 30;%300; % Number of samples in each Distribution
    npts2 = 30;
    npts3 = 180;
    npts4 = 40;
    data = [randn(2,npts1)*0.4 randn(2,npts2)*0.4+[2; 0.0]*ones(1,npts2)...
        randn(2,npts3)*1.1+[1.0; -3.5]*ones(1,npts3) randn(2,npts4)*0.6+[5.5; -6.0]*ones(1,npts4) ]; % Create Dataset
    truelabels = [ ones(1,npts1) 2*ones(1,npts2) 3*ones(1,npts3) 4*ones(1,npts4)];
  
elseif caseid == 2,
    npts = 100;
    [pts1] = randcrescent(npts,5,pi,[3 1],1, 1.8);
    [pts2] = randcrescent(npts,25,-2*pi,[-9 9],1, 2.5);
    [pts3] = randcrescent(npts,10,-3/2*pi,[-10 11],1, 1.8);
    [pts4] = randcrescent(npts,10,-0.5*pi,[-20 10],1, 1.8);
    [pts5] = randcrescent(npts,20,-pi,[-20 10],1, 2);
    data = [pts1 pts2 pts3 pts4 pts5];

    size_cluster = [ npts npts npts npts npts];
    truelabels = [ones(1,npts), ones(1,npts)*2,...
       ones(1,npts)*3, ones(1,npts)*4, ones(1,npts)*5];
        
elseif caseid == 3,
    num_cluster = 5;
    npts1 = 30;    npts2 = 150;    npts3 = 30;    npts4 = 200;    npts5 = 30;
    radius2 = 20;    radius4 = 30;

    truelabels = [ones(1,npts1), ones(1,npts2)*2, ones(1,npts3)*3, ones(1,npts4)*4 ones(1,npts5)*5];
    r_noise = 4;

    raw_data1 = randn(2,npts1);
    center = [-7,-3]; sig = [1.8,1.8];
    data1 = [[center(1)+sig(1)*raw_data1(1,:);...
            center(2)+sig(2)*raw_data1(2,:)]];

    tmp2 = rand(2,npts2)-0.5;
    [t,idt] = sort(tmp2(2,:));    
    raw_data2 = [tmp2(1,idt)*r_noise;...
     tmp2(2,idt)*2];
    data2 = [(radius2-raw_data2(1,:)).*...
        cos(pi*raw_data2(2,:));...
        (radius2-raw_data2(1,:)).*...
        sin(pi*raw_data2(2,:))];

    %   center = [0,0];sig = [1,2];
    % size_cluster_base

    raw_data3 = randn(2,npts3);

    center = [7,-3]; sig = [1.8,1.8];
    data3 = [[center(1)+sig(1)*raw_data3(1,:);...
            center(2)+sig(2)*raw_data3(2,:)]];

    tmp4 = rand(2,npts4)-0.5;
    [t,idt] = sort(tmp4(2,:)); 

    raw_data4 = [tmp4(1,idt)*r_noise;...
     tmp4(2,idt)*2];

    data4 = [(radius4-raw_data4(1,:)).*...
        cos(pi*raw_data4(2,:));...
        (radius4-raw_data4(1,:)).*...
        sin(pi*raw_data4(2,:))];

    raw_data5 = randn(2,npts5);
    center = [0,7]; sig = [1.8,1.8];
    data5 = [[center(1)+sig(1)*raw_data5(1,:);...
            center(2)+sig(2)*raw_data5(2,:)]];

    data = [data1 data2 data3 data4 data5];   
    
elseif caseid == 4,
    num_cluster = 5;
    npts1 = 40;   npts2 = 40;   npts3 = 40;   npts4 = 40;   npts5 = 160;

    truelabels = [ones(1,npts1), ones(1,npts2)*2, ones(1,npts3)*3, ones(1,npts4)*4 ones(1,npts5)*5];
    r_noise = 6;


    scale1 = 0.5;
    s_v = 5;
    s_h = 5;

    center = [-7,-12];
    a = [s_h*randn(1,npts1)*scale1;s_v*randn(1,npts1)*scale1];
    data1 = center'*ones(1,npts1) + a;

    center = [7,12];
    a = [s_h*randn(1,npts2)*scale1;s_v*randn(1,npts2)*scale1];
    data2 = center'*ones(1,npts2) + a;

    center = [-7,12];
    a = [s_h*randn(1,npts3)*scale1;s_v*randn(1,npts3)*scale1];
    data3 = center'*ones(1,npts3) + a;

    center = [7,-12];
    a = [s_h*randn(1,npts4)*scale1;s_v*randn(1,npts4)*scale1];
    data4 = center'*ones(1,npts4) + a;

    data5 = [];
    center = [0,0]'; sig = [1.0,1.0]';
    for x_idx= -39:40
       for y_idx = -0.5:1:0.5
           data5 = [ data5 (center + [x_idx; y_idx]+sig.*randn(2,1)) ];
       end
    end

    data = [data1 data2 data3 data4 data5];   

elseif caseid == 5,
    npts1 = 100;
    npts2 = 100;
    npts3 = 150;
    
    data1 = [ 3 * randn(2,npts1)+ [-9,0]' * ones(1,npts1) ]; % Create Dataset
    data2 = [ 3 * randn(2,npts2)+ [9,0]' * ones(1,npts2) ]; % Create Dataset
    
    [data3] = randcrescent(npts3,25,-pi/2,[0 5],1.0,1.5);
    
    data = [data1 data2 data3];
    
    truelabels = [ ones(1,npts1) 2*ones(1,npts2) 3*ones(1,npts3)];
   
elseif caseid == 6,
    num_cluster = 6;
   radius = 14;
   size_cluster = [160,40,30,30,30,30];
   truelabels = [ones(1,160), ones(1,40)*2, ones(1,30)*3,...
            ones(1,30)*4, ones(1,30)*5, ones(1,30)*6];
        
   raw_data = randn(2,sum(size_cluster));
   tmp = rand(2,size_cluster(1))-0.5;

   [t,idt] = sort(tmp(2,:));
   r_noise = 3;
   raw_data2 = [tmp(1,idt)*r_noise;...
             tmp(2,idt)*2];

   data = [(radius-raw_data2(1,1:size_cluster(1))).*...
        cos(pi*raw_data2(2,1:size_cluster(1)));...
        (radius-raw_data2(1,1:size_cluster(1))).*...
        sin(pi*raw_data2(2,1:size_cluster(1)))];

   
   center = [0,0];sig = [2,3.5];
   % size_cluster_base
   scb = size_cluster(1)+1;
   scb_next = scb+size_cluster(2)-1;
   data = [data,[center(1)+sig(1)*raw_data(1,scb:scb_next);...
                center(2)+sig(2)*raw_data(2,scb:scb_next)]];


   center = [radius+11,8]; sig = [2.5,2.5];
   scb = scb_next+1;
   scb_next = scb+size_cluster(3)-1;
   data = [data,[center(1)+sig(1)*raw_data(1,scb:scb_next);...
                center(2)+sig(2)*raw_data(2,scb:scb_next)]];

   center = [radius+11,-8]; sig = [2.5,2.5];
   scb = scb_next+1;
   scb_next = scb+size_cluster(4)-1;
   data = [data,[center(1)+sig(1)*raw_data(1,scb:scb_next);...
                center(2)+sig(2)*raw_data(2,scb:scb_next)]];
            
   center = [radius+24,8]; sig = [2.5,2.5];
   scb = scb_next+1;
   scb_next = scb+size_cluster(3)-1;
   data = [data,[center(1)+sig(1)*raw_data(1,scb:scb_next);...
                center(2)+sig(2)*raw_data(2,scb:scb_next)]];

   center = [radius+24,-8]; sig = [2.5,2.5];
   scb = scb_next+1;
   scb_next = scb+size_cluster(4)-1;
   data = [data,[center(1)+sig(1)*raw_data(1,scb:scb_next);...
                center(2)+sig(2)*raw_data(2,scb:scb_next)]];
end

