function do_comparison_points
spRow = 3; spCol = 3; % sub-windows

bShow = 1;      % visualize the results? ( turn off at nInteration >> 1 )
nIteration = 1; % the num of trials

% Registered methods
method = registerClusteringMethods;
% methods(1).name = 'authorityShift';
% methods(2).name = 'kMeans';
% methods(3).name = 'meanShift';
% methods(4).name = 'MCL';
% methods(5).name = 'spectralkMeans';
selectedMethods = [ 1 2 3 4 5];

% param for authority-shift
iparam.authorityshift_c = 0.95;   % weight factor for PPR (between walk & jump)

% param for mean-shift
iparam.meanshift_bandwidth = 1.4; %8; 9.5; % to be tunned for each example

addpath( [ pwd '/evaluation']); addpath( [ pwd '/plot']); addpath( [ pwd '/compute_affinity' ] );
addpath( [ pwd '/alg_authorityshift' ] );   addpath( [ pwd '/alg_meanshift' ] );
addpath( [ pwd '/alg_MCL' ] );  addpath( [ pwd '/alg_spectralkmeans' ] );

tempStr = input('Input the data # (1-6): ','s');
dataIdx = str2num(tempStr);

% load data and visualize them
[ points truelabels ] = load_pointset_data(dataIdx);
nPoints = size(points,2); 
hFigSub1 = figure('Name','data');clf;
plot2Dpoints_with_clusters_AP(points);  axis equal
title(sprintf('%d points',nPoints));

% determine the num of clusters k 
tempStr = input('Number of clusters k: ','s');
kCluster = str2num(tempStr);
close(hFigSub1);
margin_ratio=0.07; % marginal space for visualization

% initialization for output
for idx = 1:length(method);
    output(idx).etime = 0; output(idx).validity = zeros(1,5);
end

%% iterative trial loops begin
for iterRun = 1:nIteration
    
    fprintf('------------------------------------------ iteration # %d\n',iterRun);
    
    %% load and show the point set
    [ points truelabels ] = load_pointset_data(dataIdx);
    nPoints = size(points,2); 
    if bShow
        minX = min(points(1,:));    maxX = max(points(1,:));
        minY = min(points(2,:));    maxY = max(points(2,:));
        marginX = (maxX-minX)*margin_ratio; marginY = (maxY-minY)*margin_ratio;
        hFigMain1 = figure('Name','data');clf;
        spIdx = 1;  subplot(spRow,spCol,spIdx,'replace');
        plot2Dpoints_with_clusters_AP(points);  axis equal
        xlim([minX-marginX maxX+marginX]);    ylim([minY-marginY maxY+marginY]);
        title(sprintf('%d points',nPoints));
        %pause;
    end

    %% Compute affinity matrices for clustering
    [Aaff, Sdist] = compute_affinity(points);
    % column-stochastic matrix
    Prw = Aaff;
    for itr = 1:nPoints, Prw(itr,itr) = 0; end
    % normalize by column
    D = diag( sum(Prw,1) );
    Prw =  Prw * D ^(-1);
    iparam.k = kCluster;
    iparam.Prw = Prw; iparam.Aaff = Aaff; iparam.Sdist = Sdist;

    %% perform various clustering algorithms on it 
    for idx = selectedMethods
        tic;
        labels = feval(method(idx).func, points, iparam );
        etime = toc;
        fprintf('%s done... %f seconds\n',method(idx).name,etime);
        if bShow
            set(0,'CurrentFigure',hFigMain1)
            spIdx = spIdx + 1; subplot(spRow,spCol,spIdx,'replace');
            plot2Dpoints_with_clusters_AP(points,labels);    axis equal
            xlim([minX-marginX maxX+marginX]);    ylim([minY-marginY maxY+marginY]);
            title(sprintf('%s %d groups (%.2fs)',method(idx).name,length(unique(labels)),etime));
        end
        
        if length(truelabels) == 0, continue;        end
        res = labels;
        
        % evaluate nomalized mutual information 
        MIhat = MutualInfo(truelabels,res);
        % evaluate other external validity indices 
        validity = valid_external(res', truelabels'); %Rand; AR; Jac; FM
        if isempty(validity)
           disp('validation error');      
        end
        % record results
        validity = [ MIhat validity' ];
        output(idx).labels = labels;
        output(idx).etime = output(idx).etime + etime;
        output(idx).validity = output(idx).validity + validity;
        fprintf('NMI:%.3f, Rand:%.3f, AR:%.3f, Jac:%.3f, FM:%.3f\n',validity(1:5));
    end
    
end %iterative trial loops end

%% summarize the average of results
if bShow, figure(hFigMain1); end;
if length(truelabels) > 0
    fprintf('\n\n');
    fprintf('---* Average Performance *---\n');
    for idx = selectedMethods
        output(idx).etime = output(idx).etime / nIteration;
        output(idx).validity = output(idx).validity./ nIteration;
        fprintf('%15s (%.2fs) -   NMI:%.3f,  Rand:%.3f,  AR:%.3f,  Jac:%.3f,  FM:%.3f\n',...
            method(idx).name, output(idx).etime, output(idx).validity(1:5));
    end
end